@@ -1,6 +1,6 @@
 # MIT License
 
-# Copyright (c) 2016-2019 Martin Beroiz
+# Copyright (c) 2016 Martin Beroiz
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -43,7 +43,7 @@ functions, seeing and atmospheric conditions.
 """
 
 
-__version__ = "2.1"
+__version__ = "2.3.1"
 
 __all__ = [
     "MIN_MATCHES_FRACTION",
@@ -59,9 +59,8 @@ __all__ = [
 
 
 import numpy as _np
-
 from skimage.transform import estimate_transform
-from skimage.transform import matrix_transform  # noqa
+from skimage.transform import matrix_transform
 
 PIXEL_TOL = 2
 """The pixel distance tolerance to assume two invariant points are the same.
@@ -198,6 +197,28 @@ class _MatchTransform:
         return error
 
 
+def _data(image):
+    if hasattr(image, "data") and isinstance(image.data, _np.ndarray):
+        return image.data
+    else:
+        return image
+
+
+def _bw(image):
+    "Return a 2D numpy array for an array of arbitrary channels"
+    if image.ndim == 2:
+        return image
+    return _np.mean(image, axis=-1)
+
+
+def _shape(image):
+    "Return a 2D shape for the image, ignoring channel info"
+    if image.ndim == 2:
+        return image.shape
+    h, w, ch = image.shape
+    return h, w
+
+
 def find_transform(
     source, target, max_control_points=50, detection_sigma=5, min_area=5
 ):
@@ -209,12 +230,12 @@ def find_transform(
     ``T.translation``, ``T.scale``, ``T.params``.
 
     Args:
-        source (array-like): Either a numpy array of the source image to be
-            transformed or an interable of (x, y) coordinates of the target
-            control points.
-        target (array-like): Either a numpy array of the target (destination)
-            image or an interable of (x, y) coordinates of the target
-            control points.
+        source (array-like): Either a NumPy, CCData or NDData array of the
+            source image to be transformed or an interable of (x, y)
+            coordinates of the target control points.
+        target (array-like): Either a NumPy, CCData or NDData array of the
+            target (destination) image or an interable of (x, y) coordinates of
+            the target control points.
         max_control_points: The maximum number of control point-sources to find
             the transformation.
         detection_sigma: Factor of background std-dev above which is considered
@@ -235,25 +256,29 @@ def find_transform(
     from scipy.spatial import KDTree
 
     try:
-        if len(source[0]) == 2:
+        if len(_data(source)[0]) == 2:
             # Assume it's a list of (x, y) pairs
             source_controlp = _np.array(source)[:max_control_points]
         else:
             # Assume it's a 2D image
             source_controlp = _find_sources(
-                source, detection_sigma=detection_sigma, min_area=min_area
+                _bw(_data(source)),
+                detection_sigma=detection_sigma,
+                min_area=min_area,
             )[:max_control_points]
     except Exception:
         raise TypeError("Input type for source not supported.")
 
     try:
-        if len(target[0]) == 2:
+        if len(_data(target)[0]) == 2:
             # Assume it's a list of (x, y) pairs
             target_controlp = _np.array(target)[:max_control_points]
         else:
             # Assume it's a 2D image
             target_controlp = _find_sources(
-                target, detection_sigma=detection_sigma, min_area=min_area
+                _bw(_data(target)),
+                detection_sigma=detection_sigma,
+                min_area=min_area,
             )[:max_control_points]
     except Exception:
         raise TypeError("Input type for target not supported.")
@@ -345,10 +370,10 @@ def apply_transform(
 
     Args:
         transform: A scikit-image ``SimilarityTransform`` object.
-        source (numpy array): A 2D numpy array of the source image to be
-            transformed.
-        target (numpy array): A 2D numpy array of the target image. Only used
-            to set the output image shape.
+        source (numpy array): A 2D NumPy, CCData or NDData array of the source
+            image to be transformed.
+        target (numpy array): A 2D NumPy, CCData or NDData array of the target
+            image. Only used to set the output image shape.
         fill_value (float): A value to fill in the areas of aligned_image
             where footprint == True.
         propagate_mask (bool): Wether to propagate the mask in source.mask
@@ -362,29 +387,24 @@ def apply_transform(
     """
     from skimage.transform import warp
 
-    if hasattr(source, "data") and isinstance(source.data, _np.ndarray):
-        source_data = source.data
-    else:
-        source_data = source
-    if hasattr(target, "data") and isinstance(target.data, _np.ndarray):
-        target_data = target.data
-    else:
-        target_data = target
+    source_data = _data(source)
+    target_shape = _data(target).shape
 
     aligned_image = warp(
         source_data,
         inverse_map=transform.inverse,
-        output_shape=target_data.shape,
+        output_shape=target_shape,
         order=3,
         mode="constant",
         cval=_np.median(source_data),
         clip=True,
         preserve_range=True,
     )
+
     footprint = warp(
-        _np.zeros(source_data.shape, dtype="float32"),
+        _np.zeros(_shape(source_data), dtype="float32"),
         inverse_map=transform.inverse,
-        output_shape=target_data.shape,
+        output_shape=target_shape,
         cval=1.0,
     )
     footprint = footprint > 0.4
@@ -395,7 +415,7 @@ def apply_transform(
             source_mask_rot = warp(
                 source_mask.astype("float32"),
                 inverse_map=transform.inverse,
-                output_shape=target_data.shape,
+                output_shape=target_shape,
                 cval=1.0,
             )
             source_mask_rot = source_mask_rot > 0.4
@@ -418,10 +438,10 @@ def register(
     """Transform ``source`` to coincide pixel to pixel with ``target``.
 
     Args:
-        source (numpy array): A 2D numpy array of the source image to be
-            transformed.
-        target (numpy array): A 2D numpy array of the target image. Used
-            to set the output image shape as well.
+        source (numpy array): A 2D NumPy, CCData or NDData array of the source
+            image to be transformed.
+        target (numpy array): A 2D NumPy, CCData or NDData array of the target
+            image. Used to set the output image shape as well.
         fill_value (float): A value to fill in the areas of aligned_image
             where footprint == True.
         propagate_mask (bool): Wether to propagate the mask in source.mask
